
#include <PalmOS.h>
#include <PalmCompatibility.h>
#include <SerialMgrOld.h>
#include <Table.h>
#include "MathLib.h"
#include "gps.h"
#include "floatmgr.h"
#include <SysEvtMgr.h>


#define PI 3.141592653589793
#define R 3963              //radius of Earth in miles
#define recordsInCategoryUnknown    0xffff
#define  kMaxTicksWithoutTime (6 * sysTicksPerSecond)
#define DBName  "jogMAN-DB"   //database name
#define DBType  'GPS'
#define AppCreator  'luke'
#define  kTicksToLoseGPS (15 * sysTicksPerSecond)

static Boolean AnalyzeDataHandleEvent(EventPtr event);
static int StartApplication(void);
static void EventLoop(void);
static void StopApplication(void);
static Boolean RecordDataHandleEvent(EventPtr event);
static void DeleteAll(void);
static Boolean ViewDataHandleEvent(EventPtr event);
static void DoubleToString(double y,CharPtr x);
static Boolean StartFormHandleEvent(EventPtr event);
static Boolean OpenFormHandleEvent(EventPtr event);
static Boolean MainMenuHandleEvent(Word menuID);
static Boolean MapFormHandleEvent(EventPtr event);
static Boolean SettingsHandleEvent(EventPtr event);
static int CompareRecordFunc (void* rec1, void* rec2,Int other,SortRecordInfoPtr rec1SortInfo,SortRecordInfoPtr rec2SortInfo, VoidHand appInfoH);
static char CategoryName[dmCategoryLength];
static Word MemosInCategory;
static Word RecordCategory;
static Word CurrentCategory;
static void RecordDataUpdate();
static double  StringToDouble(CharPtr str);


typedef struct 
{
    long time;
    double latitude1;
    double longtitude1;
    double latitude2;
    double longtitude2;
    double distance;
    double speed;
} Thing;

typedef struct
{
    UInt renamedCategories;
    char categoryLabels[dmRecNumCategories][dmCategoryLength];
    Byte categoryUniqIDs[dmRecNumCategories];
    Byte lastUniqID;
    Byte reserved1;
    Word reserved2;
} GPSAppInfoType;

typedef struct 
{
    UInt CurrentCategory;
    int recFreq;
} PreferenceType;


int recFreq=10;
UInt offTime;
static UInt CurrentRecord=0;
char dataString[12][3][15];


DmOpenRef   gTestDB;
int mapOrient=1;
WinHandle   screenBuffer;           //pointer to buffer for map
Boolean infoBoxVisible=false;

double LoLat,LoLong,HiLat,HiLong,LastLat,LastLong,ratio;
double speed,distance;
long time;
long startTime;

int gpsStatus;
int SatNum;


PreferenceType prefs;  



static GPSAppInfoType DefaultAppInfo =
{
0x0006,
"Unfiled","",
"","",
"","",
"","",
"","",
"","",
"","",
"","",
0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,

15,

0,0
};
  




UInt  gSerialRefNum=0;
char  gSerialBuffer[3000];  // should be more than enough for one second of 

                          // data--10 lines @ 80 chars per line

int Recording;

Err err;
char recFreqStr[20];

/*---------------------------------------------------------------------------
SetFieldTextFromHandle
 ---------------------------------------------------------------------------*/
static FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH)
{
  Handle oldTxtH;
  FormPtr    frm=FrmGetActiveForm();
  FieldPtr   fldP;
  
  fldP   = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm, fieldID));
  ErrNonFatalDisplayIf(!fldP,"missing field");
  oldTxtH = FldGetTextHandle(fldP);
  FldSetTextHandle(fldP, txtH);
  FldDrawField(fldP);
 
  if (oldTxtH)
    MemHandleFree((VoidHand) oldTxtH);
  return fldP;
}


/*---------------------------------------------------------------------------
SetFieldTextFromStr
 ---------------------------------------------------------------------------*/
static FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP)
{
  VoidHand txtH;

  txtH = MemHandleNew(StrLen(strP) + 1);
  if (!txtH)
    return NULL;
    
  StrCopy(MemHandleLock(txtH),strP);
 
  MemHandleUnlock(txtH);
 
  return SetFieldTextFromHandle(fieldID, txtH);

}


/*---------------------------------------------------------------------------
InitAppInfo
 ---------------------------------------------------------------------------*/
static Err InitAppInfo(DmOpenRef dbP)
{
  UInt cardNo;
  VoidHand h;
  LocalID dbID;
  LocalID appInfoID;
  GPSAppInfoType *appInfoP;
    
  if (DmOpenDatabaseInfo(dbP,&dbID,NULL,NULL,&cardNo,NULL))
    return dmErrInvalidParam;
  if (DmDatabaseInfo(cardNo,dbID,NULL,NULL,NULL,NULL,NULL,NULL,NULL,&appInfoID,NULL,NULL,NULL))
    return dmErrInvalidParam;
    
  if (appInfoID==NULL)
  {
    h=DmNewHandle(dbP,sizeof DefaultAppInfo);
    if (!h) 
      return dmErrMemError;
            
    appInfoID=MemHandleToLocalID(h);
    DmSetDatabaseInfo(cardNo,dbID,NULL,NULL,NULL,NULL,NULL,NULL,NULL,&appInfoID,NULL,NULL,NULL);
  }
            
  appInfoP=MemLocalIDToLockedPtr(appInfoID,cardNo);
  DmWrite(appInfoP,0,&DefaultAppInfo, sizeof DefaultAppInfo);
  MemPtrUnlock(appInfoP);
  return 0;
}
 
 

/*---------------------------------------------------------------------------
OpenOrCreateDB

If database is already present, it will open it other wise, it will create it
---------------------------------------------------------------------------*/
static Err OpenOrCreateDB(DmOpenRef *dbP, ULong type, ULong creator, ULong mode, UInt cardNo, char *name, Boolean *created)
{
  Err err;
  *created = false;
  *dbP = DmOpenDatabaseByTypeCreator(type,creator,mode);  //checks to see if database present
  
  err = DmGetLastErr();
  if (! *dbP)         //if not present
  {
    err = DmCreateDatabase(0,name,creator,type,false);  //Create database
    if (err)
      return err;
    *created = true;                            //sets flag
    *dbP = DmOpenDatabaseByTypeCreator(type, creator, mode);    
    if (! *dbP)
      return DmGetLastErr();
    InitAppInfo(gTestDB);
   
  }
  return err;
}

/*---------------------------------------------------------------------------
UnpackThing
copies things from packed to unpacked
---------------------------------------------------------------------------*/
void UnpackThing(Thing *thing, const Thing *packedthing)
{
  thing->time=packedthing->time;
  thing->longtitude1=packedthing->longtitude1;
  thing->latitude1=packedthing->latitude1;
  thing->longtitude2=packedthing->longtitude2;
  thing->latitude2=packedthing->latitude2;
  thing->distance=packedthing->distance;
  thing->speed=packedthing->speed;
};



/*---------------------------------------------------------------------------
PackThing
resize record to right length and copies things in
--------------------------------------------------------------------------*/
void PackThing(Thing *thing, VoidHand thingDBEntry)
{
  UInt    length=0;
  CharPtr s;
  UInt    offset = 0;
    
  //finds the size
  length = sizeof(thing->time) +sizeof(speed)+4* sizeof(thing->latitude1) +sizeof(thing->distance);
  if (MemHandleResize(thingDBEntry, length) == 0) 
  { 
    offset=0;
    //locks the record
    s = MemHandleLock(thingDBEntry);
    DmWrite(s, offset,&thing->time,sizeof(thing->time));
    offset=sizeof(thing->time);
    DmWrite(s, offset,&thing->latitude1,sizeof(thing->latitude1));
    offset=offset+sizeof(thing->latitude1);
    DmWrite(s, offset,&thing->longtitude1,sizeof(thing->longtitude1));
    offset=offset+sizeof(thing->longtitude1);
    DmWrite(s, offset,&thing->latitude2,sizeof(thing->latitude2));
    offset=offset+sizeof(thing->latitude2);
    DmWrite(s, offset,&thing->longtitude2,sizeof(thing->longtitude2));
    offset=offset+sizeof(thing->longtitude2);
    DmWrite(s, offset,&thing->distance,sizeof(thing->distance));
    offset=offset+sizeof(thing->speed);
    DmWrite(s, offset,&thing->speed,sizeof(thing->speed));
    //unlocks the record
    MemHandleUnlock(thingDBEntry);
  }
}


/*---------------------------------------------------------------------------
PilotMain
 ---------------------------------------------------------------------------*/
DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
  int error;

  if (cmd == sysAppLaunchCmdNormalLaunch)
  {
    error = StartApplication();   // Application start code
    if (error)
      return error;
    EventLoop();                    // Event loop
    StopApplication ();           // Application stop code
  }
  return 0;
}


/*---------------------------------------------------------------------------
StartApplication
 ---------------------------------------------------------------------------*/
static int StartApplication(void)
{

  UInt        mode = dmModeReadWrite;
  Boolean     created;
  Err   err;
  UInt32 ftrValue;
  UInt16 size;
   
  err = SysLibFind("Serial Library", &gSerialRefNum);
  ErrNonFatalDisplayIf(err != 0, "Can't find serial library");
  err = SerOpen(gSerialRefNum, 0, 4800);
  
  if (err != 0) 
  {
    if (err == serErrAlreadyOpen) 
    {
      FrmAlert(SerialInUseAlert);
      SerClose(gSerialRefNum);
    } 
    else
      FrmAlert(CantopenserialAlert);
    return true;
  }
  err = SerSetReceiveBuffer(gSerialRefNum, gSerialBuffer,sizeof(gSerialBuffer));
  OpenOrCreateDB(&gTestDB, DBType, AppCreator, mode,0,DBName,&created); //gets database started
  
  err = SysLibFind(MathLibName, &MathLibRef);   //startsmath lib
  if (err)
    err = SysLibLoad(LibType, MathLibCreator, &MathLibRef);
  ErrFatalDisplayIf(err, "Can't find MathLib"); // Just an example; handle it gracefully
  err = MathLibOpen(MathLibRef, MathLibVersion);
  ErrFatalDisplayIf(err, "Can't open MathLib"); 
    
  offTime=SysSetAutoOffTime(0);
  size=sizeof(PreferenceType);

    //gets saved prefernces
  if (PrefGetAppPreferencesV10('gpse', 1, &prefs, size) != NULL)
  {
    CurrentCategory=prefs.CurrentCategory; 
    recFreq=prefs.recFreq;
  }
  else
  {
    CurrentCategory=dmUnfiledCategory;
    recFreq=10;
  }
    
  screenBuffer=WinCreateOffscreenWindow(160,160,screenFormat, &error); 
  FrmGotoForm(StartForm);
  return false;
}

/*---------------------------------------------------------------------------
ViewDataFormInit
 ---------------------------------------------------------------------------*/
static void ViewDataFormInit()
{
  FormPtr form;
  TablePtr tableP;
  ScrollBarPtr scrollBarPtr;
  UInt numRows;
  UInt i;
  int numCols=3;
  int g;
  UInt recordNum = 0;
  UInt recordIndex = DmNumRecordsInCategory(gTestDB, CurrentCategory);
  RectangleType  bounds;


  RctSetRectangle (&bounds,0, 15,160,11);
  WinEraseRectangle(&bounds,0);
  WinInvertRectangle(&bounds,0);
  FntSetFont(0);  
  WinDrawInvertedChars("Speed",StrLen("Speed"),20,14);
  WinDrawInvertedChars("Distance",StrLen("Distance"),63,14);
  WinDrawInvertedChars("Time",StrLen("Time"),125,14); 

  CurrentRecord=0;
  DmSeekRecordInCategory(gTestDB,&CurrentRecord,1,dmSeekForward,CurrentCategory);
  form= FrmGetActiveForm();
  tableP=FrmGetObjectPtr(form,FrmGetObjectIndex(form,Table1017));
  numRows=TblGetNumberOfRows (tableP);
  scrollBarPtr = FrmGetObjectPtr(form,FrmGetObjectIndex(form, ScrollBar1018));

  for (i=0;i<numRows;i++)
  {
    TblSetItemStyle(tableP,i,0,3);
    TblSetItemStyle(tableP,i,1,3);
    TblSetItemStyle(tableP,i,2,3);
  }

  for (i=0;i<numCols;i++)
    TblSetColumnUsable(tableP,i,true);

  g=recordIndex-numRows;
  if (g<1)
    g=1;
  
  SclSetScrollBar (scrollBarPtr,0, 0,g, numRows);
                
}


/*---------------------------------------------------------------------------
ViewDataFormDraw
 ---------------------------------------------------------------------------*/
static void ViewDataFormDraw(int Change)
{
  FormPtr form;
    TablePtr tableP;
    ScrollBarPtr scrollBarPtr;
    UInt numRows;
    UInt i;
    
    Thing     temp;
    VoidHand    h;
    double latitude,longtitude,speed,distance;
    long time;
    int t,g;
    int numCols=3;
    UInt        index = 0;
    
    UInt theCategory = dmAllCategories;     // could be a specific category
    UInt recordIndex = DmNumRecordsInCategory(gTestDB, theCategory);
    
    //if (recordIndex<row || row<0)
    //    return 0;
    
    form= FrmGetActiveForm();
    tableP=FrmGetObjectPtr(form,FrmGetObjectIndex(form,Table1017));
    numRows=TblGetNumberOfRows (tableP);
    
    
    if (Change>0)
       DmSeekRecordInCategory(gTestDB,&CurrentRecord,Change,dmSeekForward,CurrentCategory);
     else
       DmSeekRecordInCategory(gTestDB,&CurrentRecord,abs(Change),dmSeekBackward,CurrentCategory);
     
     
     
       i=CurrentRecord;
     for ( t=0;t< numRows; t++,i++)
     {
        if (t<recordIndex)
        {
             
              h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
            if (h)
            {
                Thing *packedThing =  MemHandleLock(h);
                UnpackThing(&temp,packedThing);
                speed=temp.speed;
                distance=temp.distance;
                time=temp.time;
                MemHandleUnlock(h);
                DoubleToString(speed,&dataString[t][0][0]);
                DoubleToString(distance,&dataString[t][1][0]);
                StrIToA(&dataString[t][2][0],time);
            }  
    
        }
        else
        {
         dataString[t][0][0]='\0';
         dataString[t][1][0]='\0';
         dataString[t][2][0]='\0';
        }
        
      }
    for (i=0;i<numRows;i++)
    {
        TblSetItemPtr(tableP,i,0,&dataString[i][0][0]);
        TblSetItemPtr(tableP,i,1,&dataString[i][1][0]);
        TblSetItemPtr(tableP,i,2,&dataString[i][2][0]);
    }
    TblMarkTableInvalid (tableP);
    TblRedrawTable(tableP);
}



/*---------------------------------------------------------------------------
AnalyzeDataDraw
 ---------------------------------------------------------------------------*/
static void AnalyzeDataDraw()
{
    FormPtr        frm = FrmGetActiveForm();
    RectangleType  bounds,r;
    Word           gadgetIndex = FrmGetObjectIndex(frm,Graph);
    int            y,x;
    UInt recordIndex = DmNumRecordsInCategory(gTestDB, CurrentCategory);
    
    
    UInt        index = 0;
    VoidHand    h;
    Thing     temp;
    int         d;
    
    UInt16 i,t,size;
    double speed,totalSpeed=0,loSpeed=10000,hiSpeed=0,totalDistance=0;
    
    char tempStr[20];


   WinEraseRectangle(&bounds,0); 
    
  
  i= DmPositionInCategory(gTestDB, 0,CurrentCategory);
  i++;
     for ( t=0;t< recordIndex; t++,i++)
     {
             h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
            
            
            if (h)
            {
                Thing *packedThing =  MemHandleLock(h);
                UnpackThing(&temp,packedThing);
                speed=temp.speed;
                if (temp.distance>totalDistance)
                    totalDistance=temp.distance;
                if (speed>hiSpeed)
                    hiSpeed=speed;
                if (speed<loSpeed)
                    loSpeed=speed;
                totalSpeed=speed+totalSpeed;
                MemHandleUnlock(h);
             }  
     }
   
     
    
    FrmGetObjectBounds(frm,gadgetIndex, &bounds);           //finds bounds of gadget

    //draws lines along left, right and bottom
    WinDrawLine(bounds.topLeft.x,bounds.topLeft.y,bounds.topLeft.x,bounds.topLeft.y+bounds.extent.y);
    WinDrawLine(bounds.topLeft.x,bounds.topLeft.y+bounds.extent.y,bounds.topLeft.x+bounds.extent.x,bounds.topLeft.y+bounds.extent.y);
    WinDrawLine(bounds.topLeft.x+bounds.extent.x,bounds.topLeft.y,bounds.topLeft.x+bounds.extent.x,bounds.topLeft.y+bounds.extent.y);
    
    //draws a gray line which repersent average speed
    y=(totalSpeed/recordIndex)/hiSpeed*(bounds.extent.y);
    //y=(((totalDistance/totalTime)-loDistance)/(hiDistance-loDistance))*39;
    WinDrawGrayLine(bounds.topLeft.x+1,bounds.topLeft.y+bounds.extent.y-y,bounds.topLeft.x+bounds.extent.x-1,bounds.topLeft.y+bounds.extent.y-y);
    StrIToA(tempStr,hiSpeed);
    tempStr[5]='\0';
     FrmCopyLabel(frm,HighSpeedLabel,tempStr);
     StrCat(tempStr," MPH");
     FrmCopyLabel(frm,HighSpdLabel,tempStr);
    //finds the ratio for records to pixels 
    d=recordIndex/(bounds.extent.x-2);    
    if (d==0)
        d=1;
    d=trunc(d);    
    //draws a line for each record
     i= DmPositionInCategory(gTestDB, 0,CurrentCategory);
    i++;  
    x=bounds.topLeft.x+1; 
    for (t=1;t< recordIndex; t++,i++)
    {
          h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
            if (h && (remainder(t,d)==0))
            {
             
                Thing *packedThing = MemHandleLock(h);
                UnpackThing(&temp,packedThing);

            //finds how high things should be
                y=((temp.speed)/hiSpeed)*bounds.extent.y;
                MemHandleUnlock(h);
                WinDrawLine(x,bounds.topLeft.y+bounds.extent.y-y,x,bounds.topLeft.y+bounds.extent.y);
                x=x+1;             
           }
    }
   
   
   DoubleToString(totalSpeed/recordIndex,tempStr);
   tempStr[4]='\0';
   StrCat(tempStr," MPH");
   FrmCopyLabel(frm,AvgSpdLabel,tempStr);
   DoubleToString(totalDistance,tempStr);
   tempStr[4]='\0';
   StrCat(tempStr," Miles");
   FrmCopyLabel(frm,TotalDistLabel,tempStr);
   
   
     StrIToA(tempStr,totalDistance);
    
     FrmCopyLabel(frm,TotalDistLabel,tempStr);
     
 
   /*         
    StringDouble(DistanceStr,totalDistance);
    StringDouble(TimeStr,(double)totalTime/60);
    StringDouble(SpeedStr,(double)(totalDistance/totalTime)*3600);
    StringDouble(CalStr,(totalDistance*.653*wieght));
     */

  
}

/*---------------------------------------------------------------------------
Distance
calculates the distance between two longitudes and latitudes. all measure in radian
---------------------------------------------------------------------------*/
static double Distance(double lt1,double lg1,double lt2,double lg2)
{
    typedef struct
    {
        double x,y,z;
    } point;
    
   double      term1,term2,term3;
    point       j,k;
    double      distance;

    if ((lg1!=0)&&(lg2!=0))
    {       
        lg1 =(PI/180.0)*lg1*-1;
        lt1  =(PI/180.0)*lt1;
        lt2   =(PI/180.0)*lt2;
        lg2  =(PI/180.0)*lg2*-1;
         
        j.x=sin(lg1)*cos(lt1);
        j.y=cos(lg1)*cos(lt1);
        j.z=sin(lt1);
    
        k.x=sin(lg2)*cos(lt2);
        k.y=cos(lg2)*cos(lt2);
        k.z=sin(lt2);
        distance=acos(j.x*k.x+j.y*k.y+j.z*k.z);
        distance=distance*R;    
        return distance;
    }
    return 0;
}

/*---------------------------------------------------------------------------
MapInit
 ---------------------------------------------------------------------------*/
static void MapInit()
{
 UInt t,i;
 Err error;
 VoidHand    h;
 Thing temp;
 UInt recordIndex = DmNumRecordsInCategory(gTestDB, CurrentCategory);
 double temp1,temp2,temp3;
   FormPtr     frm= FrmGetActiveForm();
    RectangleType  bounds;
     Word        gadgetIndex = FrmGetObjectIndex(frm,Map);
    int         lengthX,lengthY;
 
 FrmGetObjectBounds(frm,gadgetIndex, &bounds);
    lengthX=bounds.extent.x;
    lengthY=bounds.extent.y;
       
  infoBoxVisible=false;     
  CurrentRecord=0;       
DmSeekRecordInCategory(gTestDB,&CurrentRecord,1,dmSeekForward,CurrentCategory);
    //for all records
    t=CurrentRecord;
    h=DmQueryNextInCategory(gTestDB,&t,CurrentCategory);
        if (h)
        {
            Thing *packedThing = MemHandleLock(h);
            UnpackThing(&temp,packedThing);
            HiLat=temp.latitude1;
            LoLat=temp.latitude1;
            HiLong=temp.longtitude1;
            LoLong=temp.longtitude1;
         
         }
    
     i=t;
    for (t=1;t< recordIndex; t++,i++)
    {
        h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
        if (h)
        {
            Thing *packedThing = MemHandleLock(h);
            UnpackThing(&temp,packedThing);
            if (temp.latitude1<LoLat)
                LoLat=temp.latitude1;
            if (temp.latitude1>HiLat)
                HiLat=temp.latitude1;
            if (temp.longtitude1<LoLong)
                LoLong=temp.longtitude1;
            if (temp.longtitude1>HiLong)
                HiLong=temp.longtitude1;
         }
         
     }
   temp1=(HiLong-LoLong)/lengthX;
    temp2=(HiLat-LoLat)/lengthY;
    if (temp1>temp2)
    {
        ratio=temp1;
        temp3=((HiLong-LoLong)-(HiLat-LoLat))/2;
        HiLat=HiLat+temp3;
        LoLat=LoLat-temp3;
        mapOrient=2;
    }    
    else
        {
        ratio=temp2;
        temp3=((HiLat-LoLat)-(HiLong-LoLong))/2;
        LoLong=LoLong-temp3;
        HiLong=HiLong+temp3;
        mapOrient=1;
         }
   
   //DmQuickSort(gTestDB, (DmComparF *) CompareRecordFunc,mapOrient);
     
    
  temp1=HiLong-LoLong;
    WinDrawLine(bounds.topLeft.x+bounds.extent.x+1,bounds.topLeft.y-1,bounds.topLeft.x+bounds.extent.x+1,bounds.topLeft.y+bounds.extent.y+1); 

}

/*---------------------------------------------------------------------------
WidgetDraw
draws map
---------------------------------------------------------------------------*/
static void MapDraw()
{
    
    double      latitude1=0;
    double      longtitude1=0;
    double      longtitude2=0;
    double      latitude2=0;
    RectangleType  bounds;
    int         lengthX,lengthY,fromx,fromy,tox,toy;
    VoidHand    h;
    Err         error;
    UInt        index = 0;
    UInt       t,i;
    int         addX,addY;
    double      distance,distlat,distlong;
    double      temp1,temp2;
    Thing     temp;
    Word ButtonIndex;
    UInt recordIndex = DmNumRecordsInCategory(gTestDB, CurrentCategory);
    FormPtr     frm= FrmGetActiveForm();
    Word        gadgetIndex = FrmGetObjectIndex(frm,Map);
     
      
    
    FrmGetObjectBounds(frm,gadgetIndex, &bounds);
    WinEraseRectangle(&bounds,0);           //clears map
    lengthX=bounds.extent.x;
    lengthY=bounds.extent.y;
    
  temp1=(HiLong-LoLong)/lengthX;
    temp2=(HiLat-LoLat)/lengthY;
    if (temp1>temp2)
    {
        ratio=temp1;
        
    }    
    else
        {
        ratio=temp2;

         }  
    //finds ratio of pixels per degree
    
    
    longtitude1=0;
    latitude1=0;
    longtitude2=0;
    latitude2=0;
    CurrentRecord=0;
    DmSeekRecordInCategory(gTestDB,&CurrentRecord,1,dmSeekForward,CurrentCategory);
    //for all records
    i=CurrentRecord;
    for (t=0;t< recordIndex; t++,i++)
    {
        h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
        if (h)
        {
            Thing *packedThing = MemHandleLock(h);
            UnpackThing(&temp,packedThing);
            latitude1=temp.latitude1;
            longtitude1=temp.longtitude1;
            latitude2=temp.latitude2;
            longtitude2=temp.longtitude2;
            MemHandleUnlock(h);  

            
            

            //figures out where point one on the map should be
                fromx=bounds.topLeft.x+trunc((longtitude2-LoLong)/ratio);
                fromy=bounds.topLeft.y+trunc((latitude2-LoLat)/ratio);

            //figures out where point two should be
                tox=bounds.topLeft.x+trunc((longtitude1-LoLong)/ratio);
                toy=bounds.topLeft.y+trunc((latitude1-LoLat)/ratio);
            
            //draws the line
                if (RctPtInRectangle (fromx,fromy,&bounds))
                  if (RctPtInRectangle (tox,toy,&bounds))  
                    WinDrawLine(fromx,fromy,tox,toy);      
                 
            
        }
   
    } 
    //copies picture of map into buffer
    //WinCopyRectangle(WinGetActiveWindow(),screenBuffer,&bounds,0,0,0);
    ButtonIndex =FrmGetObjectIndex(frm,UpButton);
    FrmShowObject(frm,ButtonIndex);
    ButtonIndex =FrmGetObjectIndex(frm,DownButton);
    FrmShowObject(frm,ButtonIndex);
    ButtonIndex =FrmGetObjectIndex(frm,LeftButton);
    FrmShowObject(frm,ButtonIndex);
    ButtonIndex =FrmGetObjectIndex(frm,RightButton);
    FrmShowObject(frm,ButtonIndex);
    }


/*---------------------------------------------------------------------------
MapMove
 ---------------------------------------------------------------------------*/
static void MapMove(int dir)
{
double amount;

if ((dir==1) ||(dir==3))
{
   amount= (HiLat-LoLat)/3;
   if (dir==1)
   {
    HiLat=HiLat-amount;
    LoLat=LoLat-amount;
   }
    else
   {  
    HiLat=HiLat+amount;
    LoLat=LoLat+amount;
   }

}
else
if ((dir==2) ||(dir==4))
{
   amount= (HiLong-LoLong)/3;
   if (dir==2)
   {
    HiLong=HiLong+amount;
    LoLong=LoLong+amount;
   }
    else
   {  
    HiLong=HiLong-amount;
    LoLong=LoLong-amount;
   }

}

MapDraw();

}

/*---------------------------------------------------------------------------
MapZoom
 ---------------------------------------------------------------------------*/

static void MapZoom(int zoom)
{
 double temp1,temp2;

 
  
 
 if (zoom==1)
 {
 temp1=(HiLat-LoLat)/4;
 temp2=(HiLong-LoLong)/4;
 HiLat=HiLat-temp1;
 LoLat=LoLat+temp1;
 HiLong=HiLong-temp2;
 LoLong=LoLong+temp2;
 }
 else
 if (zoom==2)
 {
 temp1=(HiLat-LoLat);
 temp2=(HiLong-LoLong);
 HiLat=HiLat+temp1;
 LoLat=LoLat-temp1;
 HiLong=HiLong+temp2;
 LoLong=LoLong-temp2;
 }
 
 
 MapDraw();
 
}

/*---------------------------------------------------------------------------
MapDistance
 ---------------------------------------------------------------------------*/

static double MapDistance(double lat1,double long1,double lat2, double long2)
{
 double temp;
 
 temp=sqrt((lat1-lat2)*(lat1-lat2)+(long1-long2)*(long1-long2));
 return temp;

}


static void MapDisplayInfo( int x,int y,RectangleType *mapBounds, char *line1, char *line2,char *line3)
{
 FormPtr     frm= FrmGetActiveForm();

 int width=45;
 int height=38;
 Err error;
 WinHandle DrawWinH;
 RectangleType bounds;
 static RectangleType copyBounds;
 int BoxX;
 int BoxY;
 Word labelIndex;
 
 if (infoBoxVisible)
 {
   RectangleType targetBounds;
   RctSetRectangle (&targetBounds,0,0,160,160);
 
   DrawWinH=WinGetDrawWindow();
   WinCopyRectangle (screenBuffer,DrawWinH,&targetBounds,0,0, scrCopy);
    }
 else
 {
 
 if (x+width>(mapBounds->topLeft.x+mapBounds->extent.x))
   BoxX=x-width;
 else
    BoxX=x+3;
 
 if (y+height>(mapBounds->topLeft.y+mapBounds->extent.y))
   BoxY=y-height;
 else
    BoxY=y+3;
 
 
 RctSetRectangle (&copyBounds,0,0,160,160);
 DrawWinH=WinGetDrawWindow();
 
 WinCopyRectangle (DrawWinH,screenBuffer,&copyBounds,0,0, scrCopy);
   

 WinDrawGrayLine(x-2,y,x+2,y); 
 WinDrawGrayLine(x,y-2,x,y+2);
 RctSetRectangle (&bounds,BoxX,BoxY,width-3,height-3);
FntSetFont(0);  
 WinDrawRectangle(&bounds,2);
 
 WinDrawInvertedChars(line1,StrLen(line1),BoxX+2,BoxY+2);
 WinDrawInvertedChars(line2,StrLen(line2),BoxX+2,BoxY+12);
 WinDrawInvertedChars(line3,StrLen(line3),BoxX+2,BoxY+22);
 

  }

}


/*---------------------------------------------------------------------------
MapInfo
 ---------------------------------------------------------------------------*/
static void MapInfo(int x,int y)
{
FormPtr     frm= FrmGetActiveForm();

 UInt  index = 0;
 UInt  selected=5;
 double test,longtitude,latitude,distance=1000;
 int selectedIndex,TargetX,TargetY;
 Thing temp;
 VoidHand h;
 UInt recordIndex = DmNumRecordsInCategory(gTestDB, CurrentCategory);
 UInt t,i;
   RectangleType  bounds;
    Word        gadgetIndex = FrmGetObjectIndex(frm,Map);
   double Lat;
 double Long;
 char t1[20],t2[20],t3[20];
 
 if (infoBoxVisible)
 {
 
  MapDisplayInfo(0,0,NULL,NULL,NULL,NULL);
  infoBoxVisible=false;                  
 
 }
 else
 {
 
 FrmGetObjectBounds(frm,gadgetIndex, &bounds);
 Lat=(y-bounds.topLeft.y)*ratio+LoLat;
 Long=(x-bounds.topLeft.x)*ratio+LoLong;

   temp.latitude1=Lat;
   temp.longtitude1=Long;
   
   CurrentRecord=0;
    
    i= DmPositionInCategory(gTestDB, 0,CurrentCategory);
 for (t=0;t<recordIndex;t++,i++)
 {
 h=DmQueryNextInCategory(gTestDB,&i,CurrentCategory);
   if (h)
   {
     Thing *packedThing = MemHandleLock(h);
     UnpackThing(&temp,packedThing);
     latitude=temp.latitude1;
     longtitude=temp.longtitude1;
     MemHandleUnlock(h);
      
     test=MapDistance(latitude,longtitude,Lat,Long);
    
     if (test<distance)
        {
        selected=i; 
        distance=test;
        }
   }  
 
 
 }

h=DmQueryNextInCategory(gTestDB,&selected,CurrentCategory);
   if (h)
   {
     double timeTemp;
     Thing *packedThing = MemHandleLock(h);
     UnpackThing(&temp,packedThing);
     latitude=temp.latitude1;
     longtitude=temp.longtitude1;
     DoubleToString(temp.speed,t1);
     DoubleToString(temp.distance,t2);
     timeTemp=temp.time;
     timeTemp=timeTemp/60;
     DoubleToString(timeTemp,t3);
     
     t1[4]='\0';
     StrCat(t1," MPH");
     t2[4]='\0';
     StrCat(t2," Miles");
     t3[9]='\0';
     
     MemHandleUnlock(h);
     
     }

 TargetX=bounds.topLeft.x+trunc((longtitude-LoLong)/ratio);
 TargetY=bounds.topLeft.y+trunc((latitude-LoLat)/ratio);
            
            //draws the line
                if (RctPtInRectangle (TargetX,TargetY,&bounds))
                { 
                    
                    MapDisplayInfo(TargetX,TargetY,&bounds,t1,t2,t3);
                   
                 }
infoBoxVisible=true;                 
 }
}



/*---------------------------------------------------------------------------
EventLoop
 ---------------------------------------------------------------------------*/
static void EventLoop(void)
{
    short err;
    int formID;
    FormPtr form;
    EventType event;

    do
    {
        EvtGetEvent(&event, 200);
        if (SysHandleEvent(&event))
            continue;
        if (MenuHandleEvent((void *)0, &event, &err))
            continue;
        if (event.eType == frmLoadEvent)
        {
            formID = event.data.frmLoad.formID;
            form = FrmInitForm(formID);
            FrmSetActiveForm(form);
            switch (formID)
            {
                case SettingsForm:
                    FrmSetEventHandler(form, (FormEventHandlerPtr) SettingsHandleEvent);
                break;
                case MapForm:
                    FrmSetEventHandler(form, (FormEventHandlerPtr) MapFormHandleEvent);
                break;
                case AnalyzeDataForm:
                    FrmSetEventHandler(form, (FormEventHandlerPtr) AnalyzeDataHandleEvent);
                break;
                case StartForm:
                    FrmSetEventHandler(form, (FormEventHandlerPtr) StartFormHandleEvent);
                break;
                case RecordDataForm:
                    FrmSetEventHandler(form, (FormEventHandlerPtr) RecordDataHandleEvent);
                break;
                case ViewDataForm:
                    
                    FrmSetEventHandler(form, (FormEventHandlerPtr) ViewDataHandleEvent);
                break;
            
            }
        }
        FrmDispatchEvent(&event);
    }
    while(event.eType != appStopEvent);
}


/*---------------------------------------------------------------------------
GetField
 ---------------------------------------------------------------------------*/
static Boolean GetField(const char *buffer, UInt n, char *result)
{
   int   i;
   
   // skip n commas
   for (i = 0; i < n; i++) {
      while (*buffer && *buffer != ',')
         buffer++;
      if (*buffer == '\0')
         return false;
      buffer++;
   }
   while (*buffer && *buffer != ',')
      *result++ = *buffer++;
   if (*buffer != ',')
      return false;
   *result = '\0';
   return true;
}


/*---------------------------------------------------------------------------
StopApplication
 ---------------------------------------------------------------------------*/
 static void StopApplication(void)
{
    UInt        usecount;
    Err         error;
    UInt16        size;

    //close math lib
    error = MathLibClose(MathLibRef, &usecount);
    ErrFatalDisplayIf(error, "Can't close MathLib");
    if (usecount == 0)
        SysLibRemove(MathLibRef); 
        
        DmCloseDatabase(gTestDB);
    SysSetAutoOffTime(offTime);
    SerSetReceiveBuffer(gSerialRefNum, NULL, 0);
    SerClose(gSerialRefNum);

    prefs.CurrentCategory=CurrentCategory;
    prefs.recFreq=recFreq;
    size=sizeof(PreferenceType);
    PrefSetAppPreferencesV10('gpse', 1, &prefs,size);
    WinDeleteWindow(screenBuffer,true);
}



/*---------------------------------------------------------------------------
StringToDouble
 ---------------------------------------------------------------------------*/
static double  StringToDouble(CharPtr str)
{
    char temp[20],t1[20],t2[20],t3[20],t[2];
    double first,second,middle;
    double temp1,temp2,temp3,total;
    CharPtr y;
  
    //ddmm.mmm
    StrCopy(temp,"000000000000000000");
 //latstr = 7243.123
    StrCopy(temp,str); //copies the Latitude string into a temp ver
    y=StrStr(temp,".");     //finds where the decimal is in the lat string
    *y='\0';          //replaces the decimal with an end of string
    t[1]='\0';
    t[0]=*(y+1);
    temp2=StrAToI(t)*.1;
    t[0]=*(y+2);
    temp2=StrAToI(t)*.01+temp2;
    t[0]=*(y+3);
    temp2=StrAToI(t)*.001+temp2;
    
    
    
    //temp2=StrAToI(y+1);    //converts decimal section to an int 45.667 -> 667
    
    temp3=StrAToI(temp+2);
    *(temp+2)='\0';
    temp1=StrAToI(temp);    //converts whole number to an int
    first=temp1;            //copies from int into a double
    second=temp2;
    middle=temp3;
     DoubleToString(first,t1);
     DoubleToString(middle,t2);
     DoubleToString(second,t3);
    // FrmCustomAlert(Alert1035,t1,t2,t3);
    //second=second*.001;     //places decimal in right section
    middle=middle+second;
    //first=first*.01;        //formats whole numbers correctly
    //second=second+drem(first,1);  //adds decimal part of whole number to decimal
    //first=trunc(first);         //removes decimal part from whole number
    //second=second/.6;     //converts from minutes to degrees
    middle=middle/.6;
    middle=middle*.01;
    total=middle+first; //combines 
    return total;
}

/*---------------------------------------------------------------------------
AddRecord
 ---------------------------------------------------------------------------*/
void AddRecord(double latitude1, double longtitude1,double latitude2,double longtitude2, long time, double speed, double distance)
{
  Thing record;
  VoidHand    h;
  UInt        index = dmMaxRecordIndex;       //number of records
  UInt          attr;
    

    h = DmNewRecord(gTestDB, &index, sizeof record);    //gets a pointer to a new record
    if (h)
    {
        record.longtitude1=longtitude1;
        record.latitude1=latitude1;
        record.longtitude2=longtitude2;
        record.latitude2=latitude2;
        record.time=time;
        record.speed=speed;
        record.distance=distance;
        PackThing(&record, h);
        DmRecordInfo(gTestDB,index,&attr,NULL,NULL);
        attr &=~dmAllCategories;
        attr |=CurrentCategory;
        DmSetRecordInfo(gTestDB,index,&attr,NULL);
        DmReleaseRecord(gTestDB, index, true);
        
    }

}

/*---------------------------------------------------------------------------
StringToSecond
 ---------------------------------------------------------------------------*/     
static long StringToSecond(char *s)
{
 char temp[3];
 long total;
 
 
 temp[0]=s[2];
 temp[1]=s[3];
 temp[2]='\0';
 total=60*StrAToI(temp);

 temp[0]=s[0];
 temp[1]=s[1];
 total=total+3600*StrAToI(temp); 
 
 temp[0]=s[4];
 temp[1]=s[5];
 total=total+StrAToI(temp);

 return total;
}


/*---------------------------------------------------------------------------
DeleteAll
Removes all the records
 ---------------------------------------------------------------------------*/
static void DeleteAll(void)
{
    ULong   recordIndex;
    UInt    t;
    char    test[4];

    DmDatabaseSize(0,DmFindDatabase(0,DBName),&recordIndex,NULL,NULL);  //gets total number of recs
    for (t=1;t<=recordIndex; t++)                       //for all records
    {
        ErrNonFatalDisplayIf(DmRemoveRecord(gTestDB,0),test); //removes the first record
    }
}


/*---------------------------------------------------------------------------
StringDouble
converts double to a string
---------------------------------------------------------------------------*/
static void DoubleToString(double y,CharPtr x)
{
    char first[10];
    char second[10];
    long big;
    int small;
    double temp;
    int a1,a2,a3,a4,a5;
 
    temp=trunc(y);
    y=(y-temp);
    small=temp;
 
    a1=trunc(y/.1);
    y=y-a1*.1;
    a2=trunc(y/.01);
    y=y-a2*.01;
    a3=trunc(y/.001);
    y=y-a3*.001;
    a4=trunc(y/.0001);
    y=y-a4*.0001;
    a5=trunc(y/.00001);

    StrIToA(first,small);
    StrCopy(x,first);
    StrCat(x,".");
    StrIToA(second,a1);
    StrCat(x,second);
    StrIToA(second,a2);
    StrCat(x,second);
    StrIToA(second,a3);
    StrCat(x,second);
    StrIToA(second,a4);
    StrCat(x,second);
    StrIToA(second,a5);
    StrCat(x,second);
} 

/*---------------------------------------------------------------------------
RxData
 ---------------------------------------------------------------------------*/

static void RxData(void)
{
   double timeDif;
   double distanceTemp;
   double timeTemp;       
   double longtitude,latitude;   
     FormPtr  frm = FrmGetActiveForm();
    char temp[20];        
        ULong numBytesPending;
        
      char     theData[200];
      
   SerReceiveFlush(gSerialRefNum, 1);
        
     // throw away anything in the, 
                                       // buffer-- we want fresh data
   // we loop until an event occurs, or until
   // we update the display
   do {  
      // two lines (80 chars with <CR><LF>
      // We'll fill our read buffer, or 1/2 second between
      // bytes, whichever comes first.
      err = SerReceiveWait(gSerialRefNum, sizeof(theData) - 1, 300);
      
      if (err == serErrLineErr) {
         SerReceiveFlush(gSerialRefNum, 1);  // will clear the error
         gpsStatus=0;
         //continue;         // go back and try reading again
      } 
     /* if (err != serErrTimeOut)
         ErrFatalDisplayIf(err != 0, "SerReceiveWait");
       */
      err = SerReceiveCheck(gSerialRefNum, &numBytesPending);
      
      if (err == serErrLineErr) {
         SerReceiveFlush(gSerialRefNum, 1);  // will clear the error
         gpsStatus=0;
         //continue;         // go back and try reading again
      }
    
      if (numBytesPending > 0) {
         ULong    numBytes;
         char     *startOfMessage;
         int cap;
         // read however many bytes are waiting
         
            if (numBytes>sizeof(theData))
        cap=sizeof(theData)-1;
      else
        cap=numBytes;
        
         numBytes = SerReceive(gSerialRefNum, theData,cap, 0, &err);
        /* if (err == serErrLineErr) {
            SerReceiveFlush(gSerialRefNum, 1);  // will clear the error
            gpsStatus=0;
            continue;      // go back and try reading again
         } */
         theData[numBytes] = '\0';  // null-terminate theData
           //FrmCopyLabel(frm,MemLabel,theData);
         // look for our magic string
    
         
        //FrmCustomAlert (Alert1035,theData,NULL,NULL);        
         
         
         if ((startOfMessage = StrStr(theData, "$GPGGA")) != NULL) {
            char s[20];
            if (GetField(startOfMessage, 7,s)) 
            {
              SatNum=StrAToI(s);
             }
           }
       
         
         //if ((startOfMessage = StrStr(theData, "$GP")) != NULL)
           
         if ((startOfMessage = StrStr(theData, "$GPRMC")) != NULL) {
            char  s[20];
            if (GetField(startOfMessage, 1, s)) {
            FormPtr  frm = FrmGetActiveForm();
               // even multiple of five seconds OR it's been at 
               // least kMaxTicksWithoutTime seconds since a display
               // That way, if we lose 11:11:35, we won't have the
               // time go from 11:11:30 to 11:11:40. Instead, it'll go
               // 11:11:30, 11:11:36, 11:11:40
               //if (s[5] == '0' || s[5] == '5' ||(now - gLastTimeDisplay) > kMaxTicksWithoutTime) {
                  
                  // change from HHMMSS to HH:MM:SS
                  timeTemp=StringToSecond(s);
                  if (time!=0)
                    timeDif=timeTemp-time;
                  else
                    {
                    time=timeTemp;
                    startTime=timeTemp;
                    }
                  s[8]='\0';
                  s[7]=s[5];
                  s[6]=s[4];
                  s[5]=':';
                  s[4]=s[3];
                  s[3]=s[2];
                  s[2]=':';
                  FrmCopyLabel(frm, TimeLabel, s);
                  
                  
                 
                  if (GetField(startOfMessage, 2, s)) {
                   // update "Lost satellite" label
                     if (s[0] == 'V' ) {
                        gpsStatus=1;
                       } else if (s[0] == 'A' ) {
                        gpsStatus=2;
                     } 
                     
                     // update Lat & Lon
                     if (s[0] != 'V')  {
                        // 4 is N or S for Lat direction, 3 is lat
                        if (GetField(startOfMessage, 3, s))
                          {
                           latitude=StringToDouble(s);
                          
                          
                          FrmCopyLabel(frm, LatLabel, s);
                          
                          
                          }
                          // 5 is E or W for Lat direction, 6 is lon
                        if (GetField(startOfMessage, 5, s))
                          
                          {
                            FrmCopyLabel(frm, LongLabel, s);
                            longtitude=StringToDouble(s);
                          
                          
                          
                         
                         
                         if (Recording)
                         {
                          if (LastLat!=0 && timeDif>recFreq)
                          {
                            time=timeTemp;
                          distanceTemp=Distance(latitude,longtitude,LastLat,LastLong);
                          distance=distanceTemp+distance;
                          speed=(distanceTemp/timeDif)*3600;
                          DoubleToString(distance,s);
                          s[4]='\0';
                          FrmCopyLabel(frm, DistLabel, s);
                          DoubleToString(speed,s);
                          s[4]='\0';
                          FrmCopyLabel(frm, SpeedLabel, s);
                          DoubleToString(distance/(time-startTime),s);
                          s[4]='\0';
                          FrmCopyLabel(frm, AvgSpeedLabel, s);
                          
                            AddRecord(latitude,longtitude,LastLat,LastLong,time-startTime,speed,distance);  
                            
                            LastLat=latitude;
                            LastLong=longtitude;
                          }
                          if (LastLat==0)
                          {
                           LastLat=latitude;
                           LastLong=longtitude;
                          }
                         }
                     
                          }
                     
                     } 
                  }
                  
            }     
         } 
               
      }
     
      RecordDataUpdate(); 
   } while (!EvtSysEventAvail(false));


}
   

/*---------------------------------------------------------------------------
SelectCategory
 ---------------------------------------------------------------------------*/
static void SelectCategory(Word Trigger, Word List)
{
UInt attr;
FormPtr frm;
UInt category;
Boolean categorySelected;
Boolean categoryEdited;
char CategoryName[16];

category=CurrentCategory;

frm=FrmGetActiveForm();
categoryEdited=CategorySelect(gTestDB,frm,Trigger,List,false,&category,CategoryName,NULL,0);

categorySelected=(category!=CurrentCategory);
CurrentCategory=category;


}

/*---------------------------------------------------------------------------
MainMenuHandleEvent
 ---------------------------------------------------------------------------*/
static Boolean MainMenuHandleEvent(Word menuID)
{
Boolean handled = false;
char test[10];


switch (menuID)
{

    case AnalyzeItem:
        FrmGotoForm(AnalyzeDataForm);
        handled = true;
    break;
    case ListItem:
        FrmGotoForm(ViewDataForm);
        handled = true;
    break;
    case MapItem:
        FrmGotoForm(MapForm);
        handled = true;
    break;
    case RecordItem:
        FrmGotoForm(RecordDataForm);
        handled = true;
    break;
    case ManageItem:
        CategoryEdit(gTestDB,&CurrentCategory,CategoryTitleString,0);
        handled = true;
    break;
    case SettingsItem:
        FrmGotoForm(SettingsForm);
        handled = true;
    break;
  
}
return handled;

}


/*---------------------------------------------------------------------------
ViewDataEventHandler
 ---------------------------------------------------------------------------*/
static Boolean ViewDataHandleEvent(EventPtr event)
{
FormPtr   form;
int       handled = 0;

switch (event->eType)
    {
    case frmOpenEvent:
        form = FrmGetActiveForm();
        FrmDrawForm(form);
        ViewDataFormInit();
        ViewDataFormDraw(0);
        handled = 1;
        break;
        
    case menuEvent:
        handled = MainMenuHandleEvent(event->data.menu.itemID);
    break;
    case ctlSelectEvent:
        if (event->data.ctlEnter.controlID == Button1008 )
            {
            
            }
          if (event->data.ctlEnter.controlID == Trigger1040 )
        {    
         Word triggerPtr;
         SelectCategory(Trigger1040,List1041);
         form = FrmGetActiveForm();
         //FrmEraseForm(form);
         FrmDrawForm(form);
         triggerPtr=FrmGetObjectIndex(form,Trigger1040);
         //FrmShowObject(form,Trigger1040);
         ViewDataFormInit();
         ViewDataFormDraw(0);
         handled=1;
        }
        break;
        
    case sclRepeatEvent:
        //TableScrollRows(event->data.sclRepeat.newValue value);
        //FrmAlert(CantopenserialAlert);
        ViewDataFormDraw(event->data.sclRepeat.newValue - event->data.sclRepeat.value);
        
        handled = false;  // scrollbar needs to handle the event, too
   break;
    case nilEvent:
        handled = 1;
        
   break; 
    }
return handled;
}


/*---------------------------------------------------------------------------
MapFormHandleEvent
 ---------------------------------------------------------------------------*/
static Boolean MapFormHandleEvent(EventPtr event)
{
    FormPtr     form;
    int         handled = 0;
    Err         error;
    static int      zoom;
 
    switch (event->eType)
    {
    
        
        case menuEvent:
          handled = MainMenuHandleEvent(event->data.menu.itemID);
        break;
        case frmOpenEvent:
            form = FrmGetActiveForm();
            FrmDrawForm(form);
            //LevelDraw(50);
            MapInit();
            MapDraw();
            zoom=0;
            handled = 1;
        break;
        
        case ctlSelectEvent:
        if (event->data.ctlEnter.controlID == UpButton )
            {
            MapMove(1);
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == RightButton )
            {
            MapMove(2);
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == DownButton )
            {
            MapMove(3);
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == LeftButton )
            {
            MapMove(4);
            infoBoxVisible=false;
            handled = 1;
            }
        
        if (event->data.ctlEnter.controlID == ZoomInButton )
            {
            MapZoom(1);
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == ZoomOutButton )
            {
            MapZoom(2);
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == RefreshButton )
            {
            MapInit();
            MapDraw();
            infoBoxVisible=false;
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == Trigger1062 )
        {    
         
         SelectCategory(Trigger1062,List1063);
         MapInit();
         MapDraw();
         infoBoxVisible=false;
         handled=1;
        }
        break;
        
        case penDownEvent:
        {
         
     
         FormPtr form=FrmGetActiveForm();
         RectangleType mapbounds;
         RectangleType upbounds;
         RectangleType downbounds;
         RectangleType leftbounds;
         RectangleType rightbounds;        
         
         int x=event->screenX;
         int y=event->screenY;         
         
         FrmGetObjectBounds(form,FrmGetObjectIndex(form,Map),&mapbounds);
         FrmGetObjectBounds(form,FrmGetObjectIndex(form,UpButton),&upbounds);
         FrmGetObjectBounds(form,FrmGetObjectIndex(form,DownButton),&downbounds);
         FrmGetObjectBounds(form,FrmGetObjectIndex(form,LeftButton),&leftbounds);
         FrmGetObjectBounds(form,FrmGetObjectIndex(form,RightButton),&rightbounds);
         
         
         
         if (RctPtInRectangle(x,y,&mapbounds))
           if (!RctPtInRectangle(x,y,&upbounds))
            if (!RctPtInRectangle(x,y,&downbounds))
              if (!RctPtInRectangle(x,y,&leftbounds))
                 if (!RctPtInRectangle(x,y,&rightbounds))
                  {
                   MapInfo(x,y);
                   handled=true;
                  }
         }
         break;
           
        case nilEvent:
        { 
            handled = 1;
        }
        break;
  
    }
    return handled;
}

/*---------------------------------------------------------------------------
SettingsHandleEvent
 ---------------------------------------------------------------------------*/
static Boolean SettingsHandleEvent(EventPtr event)
{
FormPtr     form=FrmGetActiveForm();
FieldPtr    recFreqPtr;
int       handled = 0;

switch (event->eType)
    {
    case frmOpenEvent:
        form = FrmGetActiveForm();
        StrIToA(recFreqStr,recFreq);
        FrmDrawForm(form);
        SetFieldTextFromStr(recFreqField,recFreqStr);
        handled = 1;
        break;
   
   
    case ctlSelectEvent:
       
        if (event->data.ctlEnter.controlID == OKButton )
        {    
         recFreqPtr=(FieldPtr)(FrmGetObjectPtr(form, (FrmGetObjectIndex(form, recFreqField))));
         recFreq=StrAToI(FldGetTextPtr(recFreqPtr));
                       
         FrmGotoForm(StartForm);
         handled=1;
        } else
        if (event->data.ctlEnter.controlID == CancelButton )
        {    
         FrmGotoForm(StartForm);
         handled=1;
        }
        break;
    case menuEvent:
        handled = MainMenuHandleEvent(event->data.menu.itemID);
    break;
    case nilEvent:
        handled = 1;
        
   break; 
    }
return handled;
}


/*---------------------------------------------------------------------------
AnalyzeDataHandleEvent
 ---------------------------------------------------------------------------*/

static Boolean AnalyzeDataHandleEvent(EventPtr event)
{
FormPtr   form;
int       handled = 0;

switch (event->eType)
    {
    case frmOpenEvent:
        form = FrmGetActiveForm();
        FrmDrawForm(form);
        AnalyzeDataDraw();
        handled = 1;
        break;
   
   
    case ctlSelectEvent:
       
        if (event->data.ctlEnter.controlID == Trigger1079 )
        {    
         
         SelectCategory(Trigger1079,List1080);
         AnalyzeDataDraw();
         handled=1;
        }
        break;
    case menuEvent:
        handled = MainMenuHandleEvent(event->data.menu.itemID);
    break;
    case nilEvent:
        handled = 1;
        
   break; 
    }
return handled;
}
/*---------------------------------------------------------------------------
StartFormHandleEvent
 ---------------------------------------------------------------------------*/

static Boolean StartFormHandleEvent(EventPtr event)
{
FormPtr   form;
int       handled = 0;

switch (event->eType)
    {
    case frmOpenEvent:
        form = FrmGetActiveForm();
        FrmDrawForm(form);
        handled = 1;
        break;
   
   
    case ctlSelectEvent:
        if (event->data.ctlEnter.controlID == Button1008 )
            {
            //SerSetReceiveBuffer(gSerialRefNum, NULL, 0);
            //SerClose(gSerialRefNum);
                    
            FrmGotoForm(ViewDataForm);
            handled = 1;
            }
        else
        if (event->data.ctlEnter.controlID == Trigger1036 )
        {    
         
         SelectCategory(Trigger1036,List1037);
         handled=1;
        }
        break;
    case menuEvent:
        handled = MainMenuHandleEvent(event->data.menu.itemID);
    break;
    case nilEvent:
        handled = 1;
        //RxData();
   break; 
    }
return handled;
}
/*---------------------------------------------------------------------------
 RecordCheck
 ---------------------------------------------------------------------------*/
static Boolean RecordCheck()
{
   char temp[16]; 
    CategoryGetName (gTestDB, CurrentCategory,temp);
    if (FrmCustomAlert (StartRecordAlert,temp,NULL,NULL))
      return 1;
    DmDeleteCategory (gTestDB,CurrentCategory);

   return 0;
}

/*---------------------------------------------------------------------------
 RecordDataInit
 ---------------------------------------------------------------------------*/
static void RecordDataInit()
{

RectangleType  bounds;

RctSetRectangle (&bounds,0, 145,160,15);
WinInvertRectangle(&bounds,0);
WinDrawLine(5,85,155,85);
SatNum=-1;
LastLong=0;
LastLat=0;
gpsStatus=0;
Recording=0;
time=0;
distance=0;
infoBoxVisible=false;
}


/*---------------------------------------------------------------------------
 RecordDataHandleEvent
 ---------------------------------------------------------------------------*/
static void RecordDataUpdate()
{
VoidHand h;
BitmapPtr bitmap;
FormPtr form;
RectangleType  bounds;
char sat[20];

RctSetRectangle (&bounds,0, 145,160,15);
WinEraseRectangle(&bounds,0);

form = FrmGetActiveForm();
 switch (gpsStatus)
 {
    case 0:
    h=DmGetResource(bitmapRsc,QuestionBitmap); 
    FrmCopyLabel(form, LinkStatusLabel,"No GPS Link");
    break;
    case 1:
    h=DmGetResource(bitmapRsc,LinkBitmap); 
    FrmCopyLabel(form, LinkStatusLabel,"GPS Link");
    break;
    case 2:
    h=DmGetResource(bitmapRsc,DishBitmap);
    FrmCopyLabel(form, LinkStatusLabel,"GPS Data");
    break;
   }
bitmap=MemHandleLock(h);
WinDrawBitmap(bitmap,4,147);
MemHandleUnlock(h);
DmReleaseResource(h);
if (SatNum>=0)
{
    StrIToA(sat,SatNum);
    StrCat(sat," Sat.");
    FrmCopyLabel(form,NumSatLabel,sat); 
}
if (Recording)
{
    ULong mem;
    StrIToA(sat,DmNumRecordsInCategory (gTestDB,CurrentCategory));
    StrCat(sat," Records");
    FrmCopyLabel(form,RecordNumLabel,sat);
    MemCardInfo (0, NULL,NULL,NULL,NULL,NULL,NULL,&mem);
    StrPrintF(sat,"%lu Bytes Free",mem);
    FrmCopyLabel(form,MemLabel,sat);
}



WinInvertRectangle(&bounds,0);
}



/*---------------------------------------------------------------------------
 RecordDataHandleEvent
 ---------------------------------------------------------------------------*/
static Boolean RecordDataHandleEvent(EventPtr event)
{
FormPtr   form;
int       handled = 0;

switch (event->eType)
    {
    case frmOpenEvent:
            
        if (RecordCheck())
            FrmGotoForm(StartForm);
        form = FrmGetActiveForm();
        
        FrmDrawForm(form);
        RecordDataInit();
        RecordDataUpdate();
        
        handled = 1;
        break;
    case ctlSelectEvent:
        if (event->data.ctlEnter.controlID == Button1008 )
            {
            if (Recording==0)
            {
              Recording=1;
              
            }
            else
            {
            
            Recording=0;
            //SerSetReceiveBuffer(gSerialRefNum, NULL, 0);
            //SerClose(gSerialRefNum);
                      
            FrmGotoForm(ViewDataForm);
            
            }
            
            handled = 1;
            
           
            }
       
        
        break;
    case nilEvent:
        handled = 1;
        
        RxData();
        //RecordDataUpdate();
   break; 
    }
return handled;
}
